@@ -1,4 +1,7 @@
-#!/usr/bin/env python2
+# -*- coding: utf-8 -*-
+# docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
+# Please refer to the git history regarding who changed what and when in this file.
+
 '''Classes implementing retrievers (message sources getmail can retrieve mail
 from).
 
@@ -19,20 +22,6 @@ Currently implemented:
   MultidropIMAPSSLRetriever
 '''
 
-__all__ = [
-    'SimplePOP3Retriever',
-    'SimplePOP3SSLRetriever',
-    'BrokenUIDLPOP3Retriever',
-    'BrokenUIDLPOP3SSLRetriever',
-    'MultidropPOP3Retriever',
-    'MultidropPOP3SSLRetriever',
-    'MultidropSDPSRetriever',
-    'SimpleIMAPRetriever',
-    'SimpleIMAPSSLRetriever',
-    'MultidropIMAPRetriever',
-    'MultidropIMAPSSLRetriever',
-]
-
 import os
 import poplib
 import imaplib
@@ -44,6 +33,20 @@ from getmailcore.utilities import *
 from getmailcore.baseclasses import *
 from getmailcore._retrieverbases import *
 
+__all__ = [
+    'BrokenUIDLPOP3Retriever',
+    'BrokenUIDLPOP3SSLRetriever',
+    'MultidropIMAPRetriever',
+    'MultidropIMAPSSLRetriever',
+    'MultidropPOP3Retriever',
+    'MultidropPOP3SSLRetriever',
+    'MultidropSDPSRetriever',
+    'SimpleIMAPRetriever',
+    'SimpleIMAPSSLRetriever',
+    'SimplePOP3Retriever',
+    'SimplePOP3SSLRetriever',
+]
+
 
 #
 # Functional classes
@@ -63,6 +66,7 @@ class SimplePOP3Retriever(POP3RetrieverBase, POP3initMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
     )
@@ -93,10 +97,11 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
 
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
-        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfInt(name='port', required=False, default=POP3_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
@@ -157,7 +162,7 @@ class BrokenUIDLPOP3RetrieverBase(POP3RetrieverBase):
                 self.msgid_by_msgnum[msgnum] = msgnum
                 self.msgsizes[msgnum] = msgsize
             self.sorted_msgnum_msgid = sorted(self.msgid_by_msgnum.items())
-        except poplib.error_proto, o:
+        except poplib.error_proto as o:
             raise getmailOperationError('POP error (%s)' % o)
         self.gotmsglist = True
 
@@ -175,6 +180,7 @@ class BrokenUIDLPOP3Retriever(BrokenUIDLPOP3RetrieverBase, POP3initMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
     )
     received_with = 'POP3'
@@ -202,10 +208,11 @@ class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
 
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
-        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfInt(name='port', required=False, default=POP3_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
@@ -244,6 +251,7 @@ class MultidropPOP3Retriever(MultidropPOP3RetrieverBase, POP3initMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
     )
@@ -274,10 +282,11 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
 
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
-        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfInt(name='port', required=False, default=POP3_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
         ConfFile(name='keyfile', required=False, default=None),
@@ -326,6 +335,7 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         # Demon apparently doesn't support APOP
         ConfBool(name='use_apop', required=False, default=False),
     )
@@ -355,7 +365,7 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
         try:
             msgnum = self._getmsgnumbyid(msgid)
             resp, lines, octets = self.conn._longcmd('*ENV %i' % msgnum)
-        except poplib.error_proto, o:
+        except poplib.error_proto as o:
             raise getmailConfigurationError(
                 'server does not support *ENV (%s)' % o
             )
@@ -383,12 +393,16 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfString(name='uid_cache', required=False, default=None),
         ConfBool(name='record_mailbox', required=False, default=True),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
         ConfBool(name='use_xoauth2', required=False, default=False),
+        ConfString(name='imap_search', required=False, default=None),
+        ConfString(name='imap_on_delete', required=False, default=None),
+        ConfBool(name='imap_id_extension', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4'
@@ -415,8 +429,6 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInstance(name='configparser', required=False),
         ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        # socket.ssl() and socket timeouts were incompatible in Python 2.3;
-        # if you have problems, comment this line out
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
@@ -427,6 +439,7 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfString(name='uid_cache', required=False, default=None),
         ConfBool(name='record_mailbox', required=False, default=True),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
@@ -440,6 +453,9 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfBool(name='use_kerberos', required=False, default=False),
         ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='ssl_cert_hostname', required=False, default=None),
+        ConfString(name='imap_search', required=False, default=None),
+        ConfString(name='imap_on_delete', required=False, default=None),
+        ConfBool(name='imap_id_extension', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -476,6 +492,7 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfString(name='uid_cache', required=False, default=None),
         ConfBool(name='record_mailbox', required=False, default=True),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
@@ -483,6 +500,9 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
         ConfBool(name='use_kerberos', required=False, default=False),
         ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='envelope_recipient'),
+        ConfString(name='imap_search', required=False, default=None),
+        ConfString(name='imap_on_delete', required=False, default=None),
+        ConfBool(name='imap_id_extension', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4'
@@ -509,8 +529,6 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInstance(name='configparser', required=False),
         ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        # socket.ssl() and socket timeouts were incompatible in Python 2.3;
-        # if you have problems, comment this line out
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
@@ -521,6 +539,7 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfString(name='uid_cache', required=False, default=None),
         ConfBool(name='record_mailbox', required=False, default=True),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
@@ -535,6 +554,9 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='envelope_recipient'),
         ConfString(name='ssl_cert_hostname', required=False, default=None),
+        ConfString(name='imap_search', required=False, default=None),
+        ConfString(name='imap_on_delete', required=False, default=None),
+        ConfBool(name='imap_id_extension', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -552,4 +574,3 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         self.log.trace()
         self.log.info('MultidropIMAPSSLRetriever(%s)' % self._confstring()
                       + os.linesep)
-
