@@ -1,34 +1,83 @@
-#!/usr/bin/env python2.3
+# -*- coding: utf-8 -*-
+# docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
+# Please refer to the git history regarding who changed what and when in this file.
+
 '''Base classes used elsewhere in the package.
 
 '''
 
-__all__ = [
-    'ConfigurableBase',
-    'ForkingBase',
-    'ConfInstance',
-    'ConfString',
-    'ConfBool',
-    'ConfInt',
-    'ConfTupleOfStrings',
-    'ConfTupleOfTupleOfStrings',
-    'ConfPassword',
-    'ConfDirectory',
-    'ConfFile',
-    'ConfMaildirPath',
-    'ConfMboxPath',
-]
-
 import sys
 import os
 import time
 import signal
 import types
-import sets
+import codecs
+from collections import namedtuple
+import tempfile
+import errno
+import multiprocessing as mp
+
+from argparse import Namespace
+import subprocess
 
 from getmailcore.exceptions import *
 import getmailcore.logging
-from getmailcore.utilities import eval_bool, expand_user_vars
+from getmailcore.utilities import *
+
+__all__ = [
+    'ConfBool',
+    'ConfDirectory',
+    'ConfFile',
+    'ConfigurableBase',
+    'ConfInstance',
+    'ConfInt',
+    'ConfMaildirPath',
+    'ConfMboxPath',
+    'ConfPassword',
+    'ConfString',
+    'ConfTupleOfStrings',
+    'ConfTupleOfTupleOfStrings',
+    'ConfTupleOfUnicode',
+    'ForkingBase',
+    'run_command',
+]
+
+#######################################
+def run_command(command, args):
+    # Simple subprocess wrapper for running a command and fetching its exit
+    # status and output/stderr.
+    if args is None:
+        args = []
+    if isinstance(args, tuple):
+        args = list(args)
+
+    # Programmer sanity checks
+    assert isinstance(command, (bytes, str)), (
+        'command is %s (%s)' % (command, type(command))
+    )
+    assert isinstance(args, list), (
+        'args is %s (%s)' % (args, type(args))
+    )
+    for arg in args:
+        assert isinstance(arg, (bytes, str)), 'arg is %s (%s)' % (arg, type(arg))
+
+    with tempfile.TemporaryFile() as stdout, tempfile.TemporaryFile() as stderr:
+
+        cmd = [command] + args
+
+        try:
+            p = subprocess.Popen(cmd, stdout=stdout, stderr=stderr)
+        except OSError as o:
+            if o.errno == errno.ENOENT:
+                # no such file, command not found
+                raise getmailConfigurationError('Program "%s" not found' % command)
+            #else:
+            raise
+
+        rc = p.wait()
+        stdout.seek(0)
+        stderr.seek(0)
+        return (rc, stdout.read().decode().strip(), stderr.read().decode().strip())
 
 #
 # Base classes
@@ -56,7 +105,7 @@ class ConfItem:
                 )
             # Use default.
             return self.default
-        if type(val) is not self.dtype and val != self.default:
+        if not isinstance(val,self.dtype) and val != self.default:
             # Got value, but not of expected type.  Try to convert.
             if self.securevalue:
                 self.log.debug('converting %s to type %s\n'
@@ -70,7 +119,7 @@ class ConfItem:
                     val = eval_bool(val)
                 else:
                     val = self.dtype(eval(val))
-            except (ValueError, SyntaxError, TypeError), o:
+            except (ValueError, SyntaxError, TypeError) as o:
                 raise getmailConfigurationError(
                     '%s: configuration value (%s) not of required type %s (%s)'
                     % (self.name, val, self.dtype, o)
@@ -79,7 +128,7 @@ class ConfItem:
 
 class ConfInstance(ConfItem):
     def __init__(self, name, default=None, required=True):
-        ConfItem.__init__(self, name, types.InstanceType, default=default,
+        ConfItem.__init__(self, name, object, default=default,
                           required=required)
 
 class ConfString(ConfItem):
@@ -104,16 +153,50 @@ class ConfTupleOfStrings(ConfString):
             if not val:
                 val = '()'
             tup = eval(val)
-            if type(tup) != tuple:
+            if not isinstance(tup, tuple):
                 raise ValueError('not a tuple')
             val = tup
-        except (ValueError, SyntaxError), o:
+        except (ValueError, SyntaxError) as o:
             raise getmailConfigurationError(
                 '%s: incorrect format (%s)' % (self.name, o)
             )
         result = [str(item) for item in val]
         return tuple(result)
 
+class ConfTupleOfUnicode(ConfString):
+    def __init__(self, name, default=None, required=True, allow_specials=()):
+        ConfString.__init__(self, name, default=default, required=required)
+        self.specials = allow_specials
+
+    def validate(self, configuration):
+        _locals = dict([(v, v) for v in self.specials])
+        val = ConfItem.validate(self, configuration)
+        try:
+            if not val:
+                val = '()'
+            tup = eval(val, {}, _locals)
+            if tup in self.specials:
+                val = [tup]
+            else:
+                if not isinstance(tup, tuple):
+                    raise ValueError('not a tuple')
+                vals = []
+                for item in tup:
+                    try:
+                        item = item.encode()
+                        vals.append(codecs.decode(item,'ascii'))
+                    except:
+                        try:
+                            vals.append(codecs.decode(item,'utf-8'))
+                        except UnicodeError as o:
+                            raise ValueError('not ascii or utf-8: %s' % item)
+                val = vals
+        except (ValueError, SyntaxError) as o:
+            raise getmailConfigurationError(
+                '%s: incorrect format (%s)' % (self.name, o)
+            )
+        return tuple(val)
+
 class ConfTupleOfTupleOfStrings(ConfString):
     def __init__(self, name, default=None, required=True):
         ConfString.__init__(self, name, default=default, required=required)
@@ -124,20 +207,20 @@ class ConfTupleOfTupleOfStrings(ConfString):
             if not val:
                 val = '()'
             tup = eval(val)
-            if type(tup) != tuple:
+            if not isinstance(tup, tuple):
                 raise ValueError('not a tuple')
             val = tup
-        except (ValueError, SyntaxError), o:
+        except (ValueError, SyntaxError) as o:
             raise getmailConfigurationError(
                 '%s: incorrect format (%s)' % (self.name, o)
             )
         for tup in val:
-            if type(tup) != tuple:
+            if not isinstance(tup, tuple):
                 raise ValueError('contained value "%s" not a tuple' % tup)
             if len(tup) != 2:
                 raise ValueError('contained value "%s" not length 2' % tup)
             for part in tup:
-                if type(part) != str:
+                if not isinstance(part,str):
                     raise ValueError('contained value "%s" has non-string part '
                                      '"%s"' % (tup, part))
 
@@ -209,18 +292,18 @@ class ConfMboxPath(ConfString):
             )
         fd = os.open(val, os.O_RDWR)
         status_old = os.fstat(fd)
-        f = os.fdopen(fd, 'r+b')
+        f = os.fdopen(fd, 'br+')
         # Check if it _is_ an mbox file.  mbox files must start with "From "
         # in their first line, or are 0-length files.
         f.seek(0, 0)
         first_line = f.readline()
-        if first_line and first_line[:5] != 'From ':
+        if first_line and first_line[:5] != b'From ':
             # Not an mbox file; abort here
             raise getmailConfigurationError('%s: not an mboxrd file' % val)
         # Reset atime and mtime
         try:
             os.utime(val, (status_old.st_atime, status_old.st_mtime))
-        except OSError, o:
+        except OSError:
             # Not root or owner; readers will not be able to reliably
             # detect new mail.  But you shouldn't be delivering to
             # other peoples' mboxes unless you're root, anyways.
@@ -241,7 +324,7 @@ class ConfigurableBase(object):
                      - name - parameter name
                      - type - a type function to compare the parameter value
                        against (i.e. str, int, bool)
-                     - default - optional default value.  If not preseent, the
+                     - default - optional default value.  If not present, the
                        parameter is required.
     '''
 
@@ -249,9 +332,8 @@ class ConfigurableBase(object):
         self.log = getmailcore.logging.Logger()
         self.log.trace()
         self.conf = {}
-        allowed_params = set([item.name for item in self._confitems])
         for (name, value) in args.items():
-            if not name in allowed_params:
+            if name not in self:
                 self.log.warning('Warning: ignoring unknown parameter "%s" '
                                  '(value: %s)\n' % (name, value))
                 continue
@@ -272,8 +354,8 @@ class ConfigurableBase(object):
             # New class-based configuration item
             self.log.trace('checking %s\n' % item.name)
             self.conf[item.name] = item.validate(self.conf)
-        unknown_params = sets.ImmutableSet(self.conf.keys()).difference(
-            sets.ImmutableSet([item.name for item in self._confitems])
+        unknown_params = frozenset(self.conf.keys()).difference(
+            frozenset([item.name for item in self._confitems])
         )
         for param in sorted(list(unknown_params), key=str.lower):
             self.log.warning('Warning: ignoring unknown parameter "%s" '
@@ -284,9 +366,7 @@ class ConfigurableBase(object):
     def _confstring(self):
         self.log.trace()
         confstring = ''
-        names = self.conf.keys()
-        names.sort()
-        for name in names:
+        for name in list(sorted(self.conf.keys())):
             if name.lower() == 'configparser':
                 continue
             if confstring:
@@ -297,7 +377,11 @@ class ConfigurableBase(object):
                 confstring += '%s="%s"' % (name, self.conf[name])
         return confstring
 
+    def __contains__(self, confitem):
+        return confitem in {item.name for item in self._confitems}
+
 #######################################
+
 class ForkingBase(object):
     '''Base class for classes which fork children and wait for them to exit.
 
@@ -306,62 +390,78 @@ class ForkingBase(object):
         log - an object of type getmailcore.logging.Logger()
 
     '''
-    def _child_handler(self, sig, stackframe):
-        self.log.trace('handler called for signal %s' % sig)
-        try:
-            pid, r = os.wait()
-        except OSError, o:
-            # No children on SIGCHLD.  Can't happen?
-            self.log.warning('handler called, but no children (%s)' % o)
-            return
-        signal.signal(signal.SIGCHLD, self.__orig_handler)
-        self.__child_pid = pid
-        self.__child_status = r
-        self.log.trace('handler reaped child %s with status %s' % (pid, r))
-        self.__child_exited = True
 
-    def _prepare_child(self):
-        self.log.trace('')
-        self.__child_exited = False
-        self.__child_pid = None
-        self.__child_status = None
-        self.__orig_handler = signal.signal(signal.SIGCHLD, self._child_handler)
+    def _wait_for_child(self, child):
+        proc = child.process
+        pid = proc.pid
+        proc.join(timeout=60)
+        # If child is still alive we joined due to timeout.
+        if (proc.is_alive()):
+            proc.terminate()
+        exitcode = proc.exitcode
+
+        if (exitcode is None):
+            raise getmailOperationError('child pid %d failed to exit' % pid)
+        if (exitcode < 0):
+            # Child killed by a signal
+            try:
+                sig = signal.Signals(-exitcode).name
+            except:
+                sig = str(-exitcode)
 
-    def _wait_for_child(self, childpid):
-        while not self.__child_exited:
-            # Could implement a maximum wait time here
-            self.log.trace('waiting for child %d' % childpid)
-            time.sleep(1.0)
-            #raise getmailDeliveryError('failed waiting for commands %s %d (%s)'
-            #                           % (self.conf['command'], childpid, o))
-        if self.__child_pid != childpid:
-            #self.log.error('got child pid %d, not %d' % (pid, childpid))
             raise getmailOperationError(
-                'got child pid %d, not %d'
-                % (self.__child_pid, childpid)
-            )
-        if os.WIFSTOPPED(self.__child_status):
-            raise getmailOperationError(
-                'child pid %d stopped by signal %d'
-                % (self.__child_pid, os.WSTOPSIG(self.__child_status))
-            )
-        if os.WIFSIGNALED(self.__child_status):
-            raise getmailOperationError(
-                'child pid %d killed by signal %d'
-                % (self.__child_pid, os.WTERMSIG(self.__child_status))
-            )
-        if not os.WIFEXITED(self.__child_status):
-            raise getmailOperationError('child pid %d failed to exit'
-                                        % self.__child_pid)
-        exitcode = os.WEXITSTATUS(self.__child_status)
+                'child pid %d killed by signal %s'
+                % (pid, sig))
 
         return exitcode
 
+    def _pipemail(self, msg, delivered_to, received, unixfrom, stdout, stderr):
+        # Write out message
+        msgfile = tempfile.TemporaryFile('bw+')
+        msgfile.write(msg.flatten(delivered_to, received, include_from=unixfrom))
+        msgfile.flush()
+        os.fsync(msgfile.fileno())
+        # Rewind
+        msgfile.seek(0)
+        # Set stdin to read from this file
+        os.dup2(msgfile.fileno(), 0)
+        # Set stdout and stderr to write to files
+        os.dup2(stdout.fileno(), 1)
+        os.dup2(stderr.fileno(), 2)
+
+    def child_replace_me(self, msg, delivered_to, received, unixfrom,
+                         stdout, stderr, args, nolog=False):
+        self._pipemail(msg, delivered_to, received, unixfrom, stdout, stderr)
+        nolog or self.log.debug('about to execl() with args %s\n' % str(args))
+        os.execl(*args)
+
+    def forkchild(self, childfun, with_out=True):
+        self.child = child = Namespace()
+        child.stdout = tempfile.TemporaryFile('bw+')
+        child.stderr = tempfile.TemporaryFile('bw+')
+        # XXX MacOS uses spawn instead of fork, which has difficulties to
+        # pickle/unpickle objects when multiprocessing. Set 'fork' mode by default
+        ctx = mp.get_context('fork')
+        child.process = ctx.Process(target=childfun, args=(child.stdout, child.stderr))
+        child.process.start()
+        child.childpid = child.process.pid
+        self.log.trace('spawned child %d\n' % child.childpid)
+        child.exitcode = self._wait_for_child(child)
+        child.stderr.seek(0)
+        child.err = child.stderr.read().strip().decode()
+        child.stdout.seek(0)
+        if with_out:
+            child.out = child.stdout.read().strip()
+        return child
+
+    def get_msginfo(self, msg):
+        msginfo = {}
+        msginfo['sender'] = msg.sender.strip()
+        if msg.recipient != None:
+            rcpnt = msg.recipient.strip()
+            msginfo['recipient'] = rcpnt
+            msginfo['domain'] = rcpnt.lower().split('@')[-1]
+            msginfo['local'] = '@'.join(rcpnt.split('@')[:-1])
+        self.log.debug('msginfo "%s"\n' % msginfo)
+        return msginfo
 
-# For Python 2.3, which lacks the sorted() builtin
-if sys.hexversion < 0x02040000:
-    def sorted(l, key=lambda x: x):
-        lst = [(key(item), item) for item in l]
-        lst.sort()
-        return [val for (unused, val) in lst]
-    __all__.append('sorted')
